/******************************************************************************
 * File: GA_Rov_UP.ino
 * Description: Основной файл
 * Created: 22 июня 2015
 * Last Modifiede: 06 августа 2015   
 * Copyright: GNU General Public License
 * Author: Alexander Hubkin
 * Email: tp-o-vin@yandex.ru
 ******************************************************************************/

/******************************************************************************
 * CodeStyle: 3  
 ******************************************************************************/

//*******************************  Библиотеки  ******************************//
#include "Arduino.h"

// Файл конфигурации.
#include "Config.h"

// Общие типы данных.
#include "Type.h"

#include "UART.h"

// Класс протокола передачи данных.
#include "Protocol.h"

// Класс работы с портами.
#include "FastIO.h"
//*******************************  /Библиотеки  *****************************//

//*********************  Объявление переменных, констант  *******************//
// Переменные для отправки данных 1 раз в сек.
uint32_t interval = 1000;
uint32_t previousMillis = 0; 

// Переменные для проверки скорости цикла.
// Переменная для хранения micros, полученного в начале loop.
uint32_t timeCycleBegin = 0;
// Переменная для хранения времени цикла.
uint32_t timeCycle = 0;
// Переменная для хранения максимального времени цикла.
uint32_t timeCycleMax = 0;
//*********************  /Объявление переменных, констант  ******************//

//*****************************  Основные функции  **************************//
// Настройка при запуске arduino.
void setup()
{   
  // Настройка дебага пина
  MC_SET_PIN_OUTPUT(LED_PIN);
  MC_WRITE_PIN(LED_PIN, LOW);

  //TODO: Проверять необходимо ли использование RS485.
  // Настройка пина управления RS485
  MC_SET_PIN_OUTPUT(RS485_PIN_MASTER);

  // Инициализируем и открываем UART порт связи с Rov
  UARTOpen(UART_PORT, UART_PORT_SPEED);
  
  // Инициализируем и открываем UART порт Debug
  UARTOpen(UART_DEBUG_PORT, UART_DEBUG_SPEED);
}

/// <summary>
/// Основной цикл.
/// </summary>
void loop()
{ 
  // Проверка скорости цикла.
  //timeCycleBegin = micros();

  // Отправляем данные каждую секунду.
  uint32_t currentMillis = millis();

  if(currentMillis - previousMillis > interval) 
  {
    previousMillis = currentMillis;

    // Устанавливаем статус
    ps2S.statuswork = 1;

    HeadCmdSend(0, 12, 1);

    SerializeStruct((uint8_t*)&ps2S, 12);

    TailCmdSend();
    
    UARTSerialize(UART_DEBUG_PORT, '9');
    UARTSendData(UART_DEBUG_PORT);
  }

  if (rovDataS.errore == 1)
  { 
    UARTSerialize(UART_DEBUG_PORT, '8');
    UARTSendData(UART_DEBUG_PORT);

    rovDataS.errore = 0;
  }

  // Получение данных.
  UARTGetData();  

#if defined(DEBUG_SPEED_CYCLE)
  timeCycle = GetDifferenceULong(timeCycleBegin, micros());

  if (timeCycle > timeCycleMax)
  {
    timeCycleMax = timeCycle;

    char textTimeCycleMax[8];
    sprintf(textTimeCycleMax, "%lu", timeCycle);

    DebugUARTSerializeString(UART_DEBUG_PORT, textTimeCycleMax);
    UARTSerialize(UART_DEBUG_PORT, '\n');
    UARTSendData(UART_DEBUG_PORT);
  }
#endif
}

//*****************************  Основные функции  **************************//

//********** Функции работы с таймерами на основе millis(), micros() ********//
/// <summary>
/// Переменная хранящее максимальное значение unsigned long
/// </summary>
#define unsignedLongMax 4294967295UL

/// <summary>
/// Переменная хранящее 1, для проверки скорости выполнения цикла
/// </summary>
#define digitalOne 1UL

/// <summary>
/// Получение разницы между 2 uint32_t
/// </summary>
uint32_t GetDifferenceULong(uint32_t BeginTime, uint32_t EndTime)
{
  if (EndTime < BeginTime)
  {
    // Защита от переполнения
    return unsignedLongMax - BeginTime + EndTime + digitalOne;
  }
  else
  {
    return EndTime - BeginTime;
  }
}
//********* /Функции работы с таймерами на основе millis(), micros() ********//

